/* FILE: nullchannel.cc             -*-Mode: c++-*-
 *
 *      Provides a null channel -- cross-platform, cross-Tcl-version
 *		equivalent of Unix's /dev/null
 *
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 1999/07/06 03:11:46 $
 * Last modified by: $Author: dgp $
 */

#include "imports.h"
#include "nullchannel.h"

extern "C" {
static int	NullClose(ClientData, Tcl_Interp *);
static int	NullInput(ClientData, char *, int, int *);
static int	NullOutput(ClientData, char *, int, int *);
static void	NullWatch(ClientData, int);
static int	NullHandle(ClientData, int, ClientData *);
}

#if (TCL_MAJOR_VERSION > 7)
static Tcl_ChannelType nullChannelType = {
    (char *)"null",             /* Type name. */
    NULL,                       /* Always non-blocking.*/
    NullClose,                  /* Close proc. */
    NullInput,                  /* Input proc. */
    NullOutput,                 /* Output proc. */
    NULL,                       /* Seek proc. */
    NULL,                       /* Set option proc. */
    NULL,                       /* Get option proc. */
    NullWatch,                  /* Watch for events.  */
    NullHandle,                 /* Get a handle from the device. */
};
#endif

static int
NullClose(ClientData, Tcl_Interp *)
{
    return 0;
}

static int
NullInput(ClientData, char *, int, int *)
{
    return 0;			/* Always return EOF */
}

static int
NullOutput(ClientData, char *, int toWrite, int *)
{
    return toWrite;		/* Report success, but don't write anything */
}

static void
NullWatch(ClientData, int)
{
}

static int
NullHandle(ClientData, int, ClientData *)
{
    return TCL_ERROR;
}

Tcl_Channel
Nullchannel_Open()
{
#if (TCL_MAJOR_VERSION > 7)
    char channelName[16 + TCL_INTEGER_SPACE];
    static int nullChannelCount = 0;

    sprintf(channelName, "null%d", nullChannelCount++);
    return Tcl_CreateChannel(&nullChannelType, channelName, NULL,
	    TCL_READABLE | TCL_WRITABLE);
#else
# if (SYSTEM_TYPE == WINDOWS)
    return Tcl_OpenFileChannel(NULL, "NUL:", "a+", 0666);
# else
#  if (SYSTEM_TYPE == UNIX)
    return Tcl_OpenFileChannel(NULL, "/dev/nul", "a+", 0666);
#  else
    return NULL;
#  endif
# endif
#endif
}

int
Nullchannel_Init(Tcl_Interp *interp)
{
// Channel creation interfacce changed incompatibly with Tcl 8 release.
// nullChannelType conforms to Tcl 8 interface.
#if (TCL_MAJOR_VERSION > 7)
    // Since we can't redirect to the channel "null", this may not
    // be all that useful :(.
    static Tcl_Channel nullChan = Tcl_CreateChannel(&nullChannelType,
	    (char *)"null", NULL, TCL_READABLE | TCL_WRITABLE);
    Tcl_RegisterChannel(interp, nullChan);
#endif

    /* Here we could register the package, once this is separated from Oc */
    return (interp ? TCL_OK : TCL_OK);
}

